@@ -7,7 +7,6 @@
  * This file is a part of Lambda-Term.
  *)
 
-open Lwt_react
 open Lwt
 
 let main () =
@@ -17,9 +16,12 @@ let main () =
   (* Create the UI. *)
   let vbox = new LTerm_widget.vbox in
   vbox#add (new LTerm_widget.label "Hello, world!");
+  vbox#add (new LTerm_widget.label "你好，世界！");
+  vbox#add (new LTerm_widget.label "ハロー・ワールド");
+  vbox#add (new LTerm_widget.label "안녕, 세계!");
   vbox#add (new LTerm_widget.label "Press escape to exit.");
   vbox#on_event (function
-                   | LTerm_event.Key { LTerm_key.code = LTerm_key.Escape } -> wakeup wakener (); true
+                   | LTerm_event.Key { LTerm_key.code = LTerm_key.Escape; _ } -> wakeup wakener (); true
                    | _ -> false);
 
   (* Run. *)
