@@ -48,9 +48,9 @@ let make_output state out =
 
 class read_line ~term ~history ~state = object(self)
   inherit LTerm_read_line.read_line ~history ()
-  inherit [Zed_utf8.t] LTerm_read_line.term term
+  inherit [Zed_string.t] LTerm_read_line.term term
 
-  method show_box = false
+  method! show_box = false
 
   initializer
     self#set_prompt (S.const (make_prompt state))
@@ -69,7 +69,8 @@ let rec loop term history state =
       | exn -> Lwt.fail exn)
   >>= function
   | Some command ->
-    let state, out = Interpreter.eval state command in
+    let command_utf8= Zed_string.to_utf8 command in
+    let state, out = Interpreter.eval state command_utf8 in
     LTerm.fprintls term (make_output state out)
     >>= fun () ->
     LTerm_history.add history command;
