@@ -11,9 +11,11 @@ open Lwt
 open LTerm_style
 open LTerm_text
 
-lwt () =
+let main () =
   if Array.length Sys.argv <> 4 then begin
-    lwt () = LTerm.eprintlf "usage: %s <red> <blue> <green>" (Filename.basename Sys.executable_name) in
+    LTerm.eprintlf "usage: %s <red> <blue> <green>"
+      (Filename.basename Sys.executable_name)
+    >>= fun () ->
     exit 2
   end else begin
     let r = int_of_string Sys.argv.(1)
@@ -22,3 +24,5 @@ lwt () =
     LTerm.printls (eval [S(Printf.sprintf "color with component (%d, %d, %d): " r g b);
                            B_fg(rgb r g b); S"example"; E_fg])
   end
+
+let () = Lwt_main.run (main ())
