@@ -7,8 +7,6 @@
  * This file is a part of Lambda-Term.
  *)
 
-let pervasives_compare= compare
-
 open CamomileLibraryDefault.Camomile
 open Lwt_react
 open LTerm_geom
@@ -17,7 +15,6 @@ open LTerm_key
 
 let return, (>>=) = Lwt.return, Lwt.(>>=)
 
-exception Interrupt
 type prompt = LTerm_text.t
 type history = Zed_string.t list
 
@@ -64,114 +61,7 @@ let zed_common_prefix = function
 let lookup word words = List.filter (fun word' -> Zed_utf8.starts_with word' word) words
 let lookup_assoc word words = List.filter (fun (word', _) -> Zed_utf8.starts_with word' word) words
 
-(* +-----------------------------------------------------------------+
-   | Actions                                                         |
-   +-----------------------------------------------------------------+ *)
-
-type action =
-  | Edit of LTerm_edit.action
-  | Interrupt_or_delete_next_char
-  | Complete
-  | Complete_bar_next
-  | Complete_bar_prev
-  | Complete_bar_first
-  | Complete_bar_last
-  | Complete_bar
-  | History_prev
-  | History_next
-  | History_search_prev
-  | History_search_next
-  | Accept
-  | Clear_screen
-  | Prev_search
-  | Next_search
-  | Cancel_search
-  | Break
-  | Suspend
-  | Edit_with_external_editor
-
-let doc_of_action = function
-  | Edit action -> LTerm_edit.doc_of_action action
-  | Interrupt_or_delete_next_char -> "interrupt if at the beginning of an empty line, or delete the next character."
-  | Complete -> "complete current input."
-  | Complete_bar_next -> "go to the next possible completion in the completion bar."
-  | Complete_bar_prev -> "go to the previous possible completion in the completion bar."
-  | Complete_bar_first -> "go to the beginning of the completion bar."
-  | Complete_bar_last -> "go to the end of the completion bar."
-  | Complete_bar -> "complete current input using the completion bar."
-  | History_prev -> "go to the previous entry of the history."
-  | History_next -> "go to the next entry of the history."
-  | History_search_prev -> "go to the previous entry of the history that matches the start of the current line."
-  | History_search_next -> "go to the next entry of the history that matches the start of the current line."
-  | Accept -> "accept the current input."
-  | Clear_screen -> "clear the screen."
-  | Prev_search -> "search backward in the history."
-  | Next_search -> "search forward in the history."
-  | Cancel_search -> "cancel search mode."
-  | Break -> "cancel edition."
-  | Suspend -> "suspend edition."
-  | Edit_with_external_editor -> "edit input with external editor command."
-
-let actions = [
-  Interrupt_or_delete_next_char, "interrupt-or-delete-next-char";
-  Complete, "complete";
-  Complete_bar_next, "complete-bar-next";
-  Complete_bar_prev, "complete-bar-prev";
-  Complete_bar_first, "complete-bar-first";
-  Complete_bar_last, "complete-bar-last";
-  Complete_bar, "complete-bar";
-  History_prev, "history-prev";
-  History_next, "history-next";
-  History_search_prev, "history-search-prev";
-  History_search_next, "history-search-next";
-  Accept, "accept";
-  Clear_screen, "clear-screen";
-  Prev_search, "prev-search";
-  Next_search, "next-search";
-  Cancel_search, "cancel-search";
-  Break, "break";
-  Suspend, "suspend";
-  Edit_with_external_editor, "edit-with-external-editor";
-]
-
-let actions_to_names = Array.of_list (List.sort (fun (a1, _) (a2, _) -> pervasives_compare a1 a2) actions)
-let names_to_actions = Array.of_list (List.sort (fun (_, n1) (_, n2) -> pervasives_compare n1 n2) actions)
-
-let action_of_name x =
-  let rec loop a b =
-    if a = b then
-      Edit (LTerm_edit.action_of_name x)
-    else
-      let c = (a + b) / 2 in
-      let action, name = Array.unsafe_get names_to_actions c in
-      match pervasives_compare x name with
-        | d when d < 0 ->
-            loop a c
-        | d when d > 0 ->
-            loop (c + 1) b
-        | _ ->
-            action
-  in
-  loop 0 (Array.length names_to_actions)
-
-let name_of_action x =
-  let rec loop a b =
-    if a = b then
-      raise Not_found
-    else
-      let c = (a + b) / 2 in
-      let action, name = Array.unsafe_get actions_to_names c in
-      match pervasives_compare x action with
-        | d when d < 0 ->
-            loop a c
-        | d when d > 0 ->
-            loop (c + 1) b
-        | _ ->
-            name
-  in
-  match x with
-    | Edit x -> LTerm_edit.name_of_action x
-    | _ -> loop 0 (Array.length actions_to_names)
+include LTerm_read_line_base
 
 module Bindings = Zed_input.Make (LTerm_key)
 
@@ -320,6 +210,9 @@ object(self)
   method clipboard = clipboard
   method macro = macro
 
+  val interrupt: exn Lwt_mvar.t= Lwt_mvar.create_empty ()
+  method interrupt= interrupt
+
   (* The event which occurs when completion need to be recomputed. *)
   val mutable completion_event = E.never
 
@@ -483,7 +376,7 @@ object(self)
 
       | Interrupt_or_delete_next_char ->
           if Zed_rope.is_empty (Zed_edit.text edit) then
-            raise Interrupt
+            Lwt.async (fun ()-> Lwt_mvar.put interrupt Interrupt)
           else
             Zed_edit.delete_next_char context
 
@@ -693,6 +586,7 @@ class virtual ['a] abstract = object
   method virtual complete : unit
   method virtual show_box : bool
   method virtual mode : mode signal
+  method virtual interrupt : exn Lwt_mvar.t
 end
 
 (* +-----------------------------------------------------------------+
@@ -858,6 +752,7 @@ let styled_newline = [|(newline, LTerm_style.none)|]
 class virtual ['a] term term =
   let size, set_size = S.create (LTerm.size term) in
   let event, set_prompt = E.create () in
+  let editor_mode, set_editor_mode = S.create LTerm_editor.Default in
   let prompt = S.switch (S.hold ~eq:( == ) (S.const default_prompt) event) in
   let key_sequence, set_key_sequence = S.create [] in
 object(self)
@@ -891,6 +786,10 @@ object(self)
 
   val mutable running = true
 
+  val vi_state= new LTerm_vi.state
+
+  val mutable vi_edit= None
+
   initializer
     completion_start <- (
       S.fold
@@ -914,6 +813,27 @@ object(self)
               size))
     )
 
+  method editor_mode = editor_mode
+
+  val mutable vi_thread= None
+
+  method vi_state= vi_state
+
+  method set_editor_mode mode =
+    set_editor_mode mode;
+    match mode with
+    | LTerm_editor.Default->
+      vi_edit <- None;
+      (match vi_thread with
+      | Some thread->
+        LTerm_vi.Concurrent.Thread.cancel thread;
+        vi_thread <- None;
+      | None-> ());
+    | LTerm_editor.Vi->
+      let _vi_edit= vi_state#vi_edit in
+      vi_edit <- Some _vi_edit;
+      self#listen_vi _vi_edit self#interrupt
+
   method key_sequence = key_sequence
 
   method completion_start = completion_start
@@ -1160,50 +1080,126 @@ object(self)
   val mutable local_bindings = Bindings.empty
   method bind keys actions = local_bindings <- Bindings.add keys actions local_bindings
 
+  method private keyseq keys=
+    match keys with
+    | []-> return (ContinueLoop [])
+    | key::tl->
+      let res =
+        match resolver with
+          | Some res -> res
+          | None ->
+            Bindings.resolver
+              [ Bindings.pack (fun x -> x) local_bindings
+              ; Bindings.pack (fun x -> x) !bindings
+              ; Bindings.pack (List.map (fun x -> Edit x)) !LTerm_edit.bindings
+              ]
+      in
+      match Bindings.resolve key res with
+        | Bindings.Accepted actions ->
+            resolver <- None;
+            set_key_sequence [];
+            self#exec ~keys:tl actions
+        | Bindings.Continue res ->
+            resolver <- Some res;
+            set_key_sequence (S.value key_sequence @ [key]);
+            return (ContinueLoop tl)
+        | Bindings.Rejected ->
+            set_key_sequence [];
+            if resolver = None then
+              match key with
+                | { control = false; meta = false; shift = false; code = Char ch } ->
+                    Zed_macro.add self#macro (Edit (LTerm_edit.Zed (Zed_edit.Insert (Zed_char.unsafe_of_uChar ch))));
+                    self#insert ch
+                | { code = Char ch; _ } when LTerm.windows term && UChar.code ch >= 32 ->
+                    (* Windows reports Shift+A for A, ... *)
+                    Zed_macro.add self#macro (Edit (LTerm_edit.Zed (Zed_edit.Insert (Zed_char.unsafe_of_uChar ch))));
+                    self#insert ch
+                | _ ->
+                    ()
+            else
+              resolver <- None;
+            return (ContinueLoop tl)
+
+  val result= Lwt_mvar.create_empty ()
+
+  method private listen_vi vi_edit exnBox=
+    let msgBox= vi_edit#action_output in
+    let rec perform_actions= function
+      | []-> return (ContinueLoop [])
+      | action::tl->
+        LTerm_vi.perform vi_edit self#context self#exec action
+        >>= function
+        | Result _ as r -> return r
+        | ContinueLoop _-> perform_actions tl
+    in
+    let rec listen ()=
+      set_key_sequence [];
+      LTerm_vi.Concurrent.MsgBox.get msgBox >>= (function
+        | Bypass keyseq->
+          let keyseq= List.map LTerm_vi.of_vi_key keyseq in
+          self#process_keys keyseq >>= (function
+            | Result r-> Lwt_mvar.put result r
+            | ContinueLoop _-> listen ()
+            )
+        | Dummy-> listen ()
+        | Vi actions->
+          perform_actions actions
+          >>= function
+          | ContinueLoop _-> listen ()
+          | Result r-> Lwt_mvar.put result r
+        )
+    in
+    let thread=
+      Lwt.catch listen
+        (fun exn-> Lwt_mvar.put exnBox exn)
+    in
+    vi_thread <- Some (thread)
+
+  method private process_keys keys=
+    self#keyseq keys >>= function
+    | Result r-> return (Result r)
+    | ContinueLoop keys->
+      match keys with
+      | []-> return (ContinueLoop [])
+      | _-> self#process_keys keys
+
   (* The main loop. *)
   method private loop =
-    LTerm.read_event term >>= fun ev ->
+    let read_event=
+      match vi_edit with
+      | Some _->
+        Lwt.pause () >>= fun ()-> Lwt.(>|=) (LTerm.read_event term) (fun ev-> Ev ev)
+      | None-> Lwt.(>|=) (LTerm.read_event term) (fun ev-> Ev ev)
+    in
+    Lwt.pick [
+      read_event;
+      Lwt.(>|=) (Lwt_mvar.take result) (fun r-> Loop_result r);
+      Lwt.(>|=) (Lwt_mvar.take self#interrupt) (fun e-> Interrupted e);
+      ]
+    >>= function
+    | Loop_result r-> return r
+    | Interrupted exn-> raise exn
+    | Ev ev->
     match ev with
       | LTerm_event.Resize size ->
           set_size size;
           self#loop
-      | LTerm_event.Key key -> begin
-          let res =
-            match resolver with
-              | Some res -> res
-              | None ->
-                Bindings.resolver
-                  [ Bindings.pack (fun x -> x) local_bindings
-                  ; Bindings.pack (fun x -> x) !bindings
-                  ; Bindings.pack (List.map (fun x -> Edit x)) !LTerm_edit.bindings
-                  ]
-          in
-          match Bindings.resolve key res with
-            | Bindings.Accepted actions ->
-                resolver <- None;
-                set_key_sequence [];
-                self#exec actions
-            | Bindings.Continue res ->
-                resolver <- Some res;
-                set_key_sequence (S.value key_sequence @ [key]);
-                self#loop
-            | Bindings.Rejected ->
-                set_key_sequence [];
-                if resolver = None then
-                  match key with
-                    | { control = false; meta = false; shift = false; code = Char ch } ->
-                        Zed_macro.add self#macro (Edit (LTerm_edit.Zed (Zed_edit.Insert (Zed_char.unsafe_of_uChar ch))));
-                        self#insert ch
-                    | { code = Char ch; _ } when LTerm.windows term && UChar.code ch >= 32 ->
-                        (* Windows reports Shift+A for A, ... *)
-                        Zed_macro.add self#macro (Edit (LTerm_edit.Zed (Zed_edit.Insert (Zed_char.unsafe_of_uChar ch))));
-                        self#insert ch
-                    | _ ->
-                        ()
-                else
-                  resolver <- None;
-                self#loop
-        end
+      | LTerm_event.Key key ->
+        (match S.value editor_mode with
+        | LTerm_editor.Default->
+          self#process_keys [key] >>= (function
+            | Result r-> return r
+            | ContinueLoop _-> self#loop)
+        | LTerm_editor.Vi->
+          match vi_edit with
+          | Some vi_edit->
+            set_key_sequence (S.value key_sequence @ [key]);
+            LTerm_vi.Concurrent.MsgBox.put vi_edit#i (LTerm_vi.of_lterm_key key) >>= fun ()->
+            self#loop
+          | None->
+            self#process_keys [key] >>= (function
+              | Result r-> return r
+              | ContinueLoop _-> self#loop ) (* falllback to the default mode *))
       | _ ->
           self#loop
 
@@ -1215,23 +1211,24 @@ object(self)
       Sys.getenv "EDITOR"
     with Not_found -> "vi"
 
-  method private exec = function
+  method private exec ?(keys= []) actions=
+    match actions with
     | Accept :: _ when S.value self#mode = Edition ->
         Zed_macro.add self#macro Accept;
-        return self#eval
+        return (Result self#eval)
     | Clear_screen :: actions ->
         Zed_macro.add self#macro Clear_screen;
         LTerm.clear_screen term >>= fun () ->
         LTerm.goto term { row = 0; col = 0 } >>= fun () ->
         displayed <- false;
         self#queue_draw_update >>= fun () ->
-        self#exec actions
+        self#exec ~keys actions
     | Edit LTerm_edit.Play_macro :: actions ->
         Zed_macro.cancel self#macro;
-        self#exec (Zed_macro.contents macro @ actions)
+        self#exec ~keys (Zed_macro.contents macro @ actions)
     | Suspend :: actions ->
         if Sys.win32 then
-          self#exec actions
+          self#exec ~keys actions
         else begin
           let is_visible = visible in
           self#hide >>= fun () ->
@@ -1254,7 +1251,7 @@ object(self)
                   return ()
           end >>= fun () ->
           (if is_visible then self#show else return ()) >>= fun () ->
-          self#exec actions
+          self#exec ~keys actions
         end
     | Edit_with_external_editor :: actions -> begin
         let is_visible = visible in
@@ -1307,13 +1304,13 @@ object(self)
         >>= fun () ->
         (if is_visible then self#show else return ())
         >>= fun () ->
-        self#exec actions
+        self#exec ~keys actions
       end
     | action :: actions ->
         self#send_action action;
-        self#exec actions
+        self#exec ~keys actions
     | [] ->
-        self#loop
+      return (ContinueLoop keys)
 
   method run =
     (* Update the size with the current size. *)
@@ -1347,8 +1344,10 @@ object(self)
     end >>= fun () ->
 
     begin
-      Lwt.finalize (fun () ->
-          Lwt.catch (fun () ->
+      Lwt.finalize
+        (fun () ->
+          Lwt.catch
+            (fun () ->
               (* Go to the beginning of line otherwise all offset
                  calculation will be false. *)
               LTerm.fprint term "\r" >>= fun () ->
