@@ -187,6 +187,9 @@ class virtual ['a] engine : ?history : history -> ?clipboard : Zed_edit.clipboar
         completion should be displayed, and when [Some msg] [msg]
         should be displayed. *)
 
+  method interrupt : exn Lwt_mvar.t
+    (** To notify an interrupt singal *)
+
   (** {6 Completion} *)
 
   method completion_words : (Zed_string.t * Zed_string.t) list signal
@@ -238,6 +241,7 @@ class virtual ['a] abstract : object
   method virtual complete : unit
   method virtual show_box : bool
   method virtual mode : mode signal
+  method virtual interrupt : exn Lwt_mvar.t
 end
 
 (** {6 Predefined classes} *)
@@ -290,6 +294,10 @@ end
 
 (** {6 Running in a terminal} *)
 
+type 'a loop_result=
+  | Result of 'a
+  | ContinueLoop of LTerm_key.t list
+
 (** Class for read-line instances running in a terminal. *)
 class virtual ['a] term : LTerm.t -> object
   inherit ['a] abstract
@@ -297,10 +305,19 @@ class virtual ['a] term : LTerm.t -> object
   method run : 'a Lwt.t
     (** Run this read-line instance. *)
 
-  method private exec : action list -> 'a Lwt.t
+  method private exec : ?keys : LTerm_key.t list -> action list -> 'a loop_result Lwt.t
     (** Executes a list of actions. Rememver to call [Zed_macro.add
         self#macro action] if you overload this method. *)
 
+  method editor_mode : LTerm_editor.mode signal
+    (** The current editor mode. *)
+
+  method set_editor_mode : LTerm_editor.mode -> unit
+    (** Set the current editor mode. *)
+
+  method vi_state : LTerm_vi.state
+    (** Get the current vi_state . *)
+
   method bind : LTerm_key.t list -> action list -> unit
 
   method draw_update : unit Lwt.t
